APP_STORE_INDEX = {
    "ipf_dora": {
        "class_path": "ipfabric_extensions.extensions.reports.dora_example:IPFDoraExample",
        "name": "DORA Reporting",
        "icon": "💶",
        "description": "Example for creating DORA-style reporting.",
    },
    "ipf_reports": {
        "name": "IPF Reports",
        "icon": "📊",
        "description": "Advanced reporting tools for IP Fabric.",
        "git_url": "https://gitlab.com/ip-fabric/integrations/scripts/ipfabric-reports",
        "slug": "ipfabric-reports",
    },
    "ipf_infoblox": {
        "name": "IPF Infoblox",
        "icon": "🗂️",
        "description": "Infoblox extension for IP Fabric. Synchronize IP Fabric's IP Address Management with Infoblox.",
        "git_url": "https://gitlab.com/ip-fabric/integrations/ipfabric-infoblox",
        "slug": "ipfabric-infoblox",
    },
    "ipf_search": {
        "class_path": "ipfabric_extensions.extensions.search.global_search:IPFGlobalSearch",
        "name": "Global Search",
        "icon": "🔍",
        "description": "Search across snapshots for a specific IP or MAC Address.",
    },
}